︠c1e52533-954b-4319-80c3-87e5c766f31ei︠
%md

# Using Machine Learning in an HR Diagram

This worksheet is a quick exploration of clustering of unlabeled data to
classify stars in a Herzsprung-Russell Diagram (HRD) for a sample set of stars.

In this simple test, two clusters of stars are identified. I think a more
sophisticated approach is needed for the other customary groups.

Clustering code is from the scikit-learn Python package.

## References

* Introduction to HRD: [The Hertzsprung-Russell Diagram](http://www.atnf.csiro.au/outreach/education/senior/astrophysics/stellarevolution_hrintro.html)
* Sloan Digital Sky Survey DR14 Projects: [The Hertzsprung-Russell Diagram](http://skyserver.sdss.org/dr14/en/proj/advanced/hr/hrhome.aspx)
* scikit-learn documentation: [Clustering](http://scikit-learn.org/stable/modules/clustering.html)
* Hubble Space Telescope: [Hertzsprung-Russell diagram animation](http://www.spacetelescope.org/videos/heic1017b/)


︡83e7d49d-060a-48c6-a405-5a9b92770755︡{"done":true,"md":"\n# Using Machine Learning in an HR Diagram\n\nThis worksheet is a quick exploration of clustering of unlabeled data to\nclassify stars in a Herzsprung-Russell Diagram (HRD) for a sample set of stars.\n\nIn this simple test, two clusters of stars are identified. I think a more\nsophisticated approach is needed for the other customary groups.\n\nClustering code is from the scikit-learn Python package.\n\n## References\n\n* Introduction to HRD: [The Hertzsprung-Russell Diagram](http://www.atnf.csiro.au/outreach/education/senior/astrophysics/stellarevolution_hrintro.html)\n* Sloan Digital Sky Survey DR14 Projects: [The Hertzsprung-Russell Diagram](http://skyserver.sdss.org/dr14/en/proj/advanced/hr/hrhome.aspx)\n* scikit-learn documentation: [Clustering](http://scikit-learn.org/stable/modules/clustering.html)\n* Hubble Space Telescope: [Hertzsprung-Russell diagram animation](http://www.spacetelescope.org/videos/heic1017b/)"}
︠a7c01b7b-fd6f-489b-b6e4-1b03a28c222as︠
%auto
%default_mode python3
︡9f803506-d8c2-4dc9-85c4-9354d71926c4︡{"done":true}︡
︠6fcf64a9-0320-4164-b76b-81150c718ee2s︠
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from sklearn import preprocessing
from sklearn.neighbors import kneighbors_graph
from sklearn.cluster import AgglomerativeClustering
︡9652dab7-1b7d-4bc8-a6f0-2f5cc0de0f50︡{"done":true}︡
︠9608b983-6cb2-48a0-aab7-7bdb5ecb54ab︠
# Import the hipparcos-2 star catalog
# ftp://cdsarc.u-strasbg.fr/pub/cats/I/311
h2colnames = [
    "HIP",  
    "Sn",   
    "So",   
    "Nc",   
    "RArad",
    "DErad",
    "Plx",  
    "pmRA", 
    "pmDE", 
    "e_RArad",
    "e_DErad",
    "e_Plx",
    "e_pmRA",
    "e_pmDE",
    "Ntr",  
    "F2",   
    "F1",   
    "var",  
    "ic",   
    "Hpmag",
    "e_Hpmag",
    "sHp",  
    "VA",   
    "B-V",  
    "e_B-V",
    "V-I", 
]
print('number of columns',len(h2colnames))
h2colspecs = [
    (1,7),
    (8,11),
    (12,13),
    (14,15),
    (16,29),
    (30,43),
    (44,51),
    (52,60),
    (61,69),
    (70,76),
    (77,83),
    (84,90),
    (91,97),
    (98,104),
    (105,108),
    (109,114),
    (115,117),
    (118,124),
    (125,129),
    (130,137),
    (138,144),
    (145,150),
    (151,152),
    (153,159),
    (160,165),
    (166,172)
]
h2cols2 = [(x-1,y-1) for (x,y) in h2colspecs]
def read_hip2(fname="hip2.dat", nrows=10):
    df = pd.read_fwf(fname, names=h2colnames, colspecs=h2cols2, index_col=0, nrows=nrows)
    return df
︡0f846110-d84d-4bbf-ab48-9c09c6b14842︡{"stdout":"number of columns 26\n"}︡{"done":true}︡
︠87c5e816-7a93-42f0-abe2-3a47de37a997︠
# create dataframe from the data
df = read_hip2(nrows=None)
︡db895dbd-ed01-4b9f-907d-af8810566c65︡{"done":true}︡
︠f1a5f7d3-01ab-47ae-b3b0-174892c894c3︠
# distance and magnitude calculations
# http://skyserver.sdss.org/dr14/en/proj/advanced/hr/hipparcos2.aspx

# compute distance from observed parallax
df = df[df['Plx'] > 0][['Hpmag','Plx','B-V']]
df['Distance'] = 1000.0/df['Plx']

# compute absolute magnitude from apparent magnitude
df['AbsMag'] =  df['Hpmag'] - 5*np.log10(df['Distance']) + 5
df.shape
︡a3014179-51d5-401d-89fd-b510a8b8a0e5︡{"stdout":"(113942, 5)"}︡{"done":true}︡
︠daccd32d-a65a-4981-9525-aa068060c6d1︠
# display values for the star Sirius
df.loc[32349]
︡7e633626-976f-4331-9e29-29392492ce10︡{"stdout":"Hpmag        -1.087600\nPlx         379.210000\nB-V           0.009000\nDistance      2.637061\nAbsMag        1.806799\nName: 32349, dtype: float64"}︡{"done":true}︡
︠22715836-7d53-47da-bb61-f33eb2f1491ds︠
# for this quick test, plot a random sample of the catalog
df3 = df.sample(n=200)
df3.shape
︡fddf7bb3-0045-4d11-ac3b-b310dd49069c︡{"stdout":"(200, 5)"}︡{"done":true}︡
︠b4509154-bfe6-4a47-a174-a0179230eba5︠
# plot absolute magnitude vs b-v color for sample set
# this is an HRD subset plot
# use agglomerative clustering to color stars in two groups
X = df3.as_matrix(columns=['AbsMag','B-V'])
scaler = preprocessing.StandardScaler().fit(X)
XT = scaler.transform(X)
connectivity = kneighbors_graph(XT, n_neighbors=4, include_self=False)
ac = AgglomerativeClustering(n_clusters=2, connectivity=connectivity).fit(XT)
df3['color'] = ac.labels_
#cm = 'bgrgrcmyk'
cm = [
    'crimson',
    'darkblue'
]
cmap = df3['color'].apply(lambda x: cm[x])
df3.plot(x='B-V', y='AbsMag', kind='scatter', figsize=(6,4),
        title='AbsMag vs. B-V', color=cmap,
        grid=True, legend=False).invert_yaxis()
︡9cc44754-4408-4b22-9b7f-55fcf785c51e︡{"file":{"filename":"/tmp/tmpXUqAKg.png","show":true,"text":null,"uuid":"96fd5ec6-f627-4043-b136-d4e2c7da0139"},"once":false}︡{"done":true}︡
︠de52b78f-2139-4b11-94d7-cdf177e58dces︠
︡48bc0691-002e-4f91-a895-f18a6838e164︡{"done":true}︡









